SystemOrganization addCategory: #'Seaside-Adaptors-Swazoo'!

WAServerAdaptor subclass: #WASwazooAdaptor
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Swazoo'!

!WASwazooAdaptor class methodsFor: 'testing' stamp: 'pmm 11/5/2008 16:38'!
isAbstract
	^ false! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:41'!
basicStart
	site port: self port.
	site start! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:44'!
basicStop
	site stop! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:53'!
contentsOfResponse: aResponse
	| contents |
	contents := aResponse contents contents.
	^aResponse contentType isBinary
		ifTrue: [ contents ]
		ifFalse: [ self encode: contents ]! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:53'!
convertCookiesFrom: aWAResonse to: aSwazooResponse
	| cookies setCookieField setCookie2Field |
	cookies := aWAResonse cookies.
	cookies isEmpty ifTrue: [ ^self ].
	
	setCookieField := WorkingHTTPSetCookieField new.
	setCookie2Field := WorkingHTTPSetCookie2Field new.

	cookies do: [ :each |
		setCookieField addCookie: each oldNetscapeString.
		setCookie2Field addCookie: each rfc2965String ].
	
	aSwazooResponse headers addField: setCookieField.
	aSwazooResponse headers addField: setCookie2Field! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:49'!
convertHeader: aSwazooRequest
	| swazooFields fields |
	swazooFields := aSwazooRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:56'!
convertRequest: aSwazooRequest
	^WARequest
		method: aSwazooRequest methodName
		url: (self urlFrom: aSwazooRequest)
		headers: (self convertHeader: aSwazooRequest)
		fields: (self fieldsOf: aSwazooRequest)
		cookies: (self cookiesFrom: aSwazooRequest)
		remoteAddress: (self remoteAddressOf: aSwazooRequest)
		nativeRequest: aSwazooRequest! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:51'!
convertResponse: aSeasideResponse
	| swazooResponse |
	aSeasideResponse isNil
		ifTrue:  [ ^self requestHandlingAborted ].
	swazooResponse := HTTPResponse new.
			
	aSeasideResponse headers do: [ :each |
		| field fieldClass |
		fieldClass := HeaderField classForFieldName: each key.
		field := fieldClass newForFieldName: each key withValueFrom: each value.
		swazooResponse headers addField: field ].
	self convertCookiesFrom: aSeasideResponse to: swazooResponse.
	
	swazooResponse code: aSeasideResponse status asNumber.
	swazooResponse entity: (self contentsOfResponse: aSeasideResponse).
	swazooResponse contentType: aSeasideResponse contentType seasideString.
	^swazooResponse! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:42'!
cookiesFrom: aSwazooRequest
	^WARequestCookie fromString: aSwazooRequest cookie! !

!WASwazooAdaptor methodsFor: 'defaults' stamp: 'pmm 11/8/2008 12:30'!
defaultCodec
	^ SeasidePlatformSupport codecForEncoding: 'utf-8'! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:56'!
fieldsOf: aSwazooRequest
	| merger |
	merger := WADictionaryMerger new.
	self getFieldsOf: aSwazooRequest into: merger.
	aSwazooRequest isPost ifTrue: [
		self postFieldsOf: aSwazooRequest into: merger ].
	^merger dictionary! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:53'!
getFieldsOf: aSwazooRequest into: aDictionaryMerger
	aSwazooRequest queries do: [ :each |
		aDictionaryMerger at: each key put: (self decodeUrl: each value) ]! !

!WASwazooAdaptor methodsFor: 'serving' stamp: 'pmm 11/6/2008 14:38'!
handleRequest: aSwazooRequest
	(self isImplemented: aSwazooRequest methodName)
		ifFalse: [ ^self notImplementedResponse ].

	^ [
		| request response |
		request := self convertRequest: aSwazooRequest.
		response := self dispatchRequest: request.
		[ self convertResponse: response ]
			ensure: [ response release ] ]
		on: Error
		do: [ :exception | 
			self internalErrorResponseFor: exception ]! !

!WASwazooAdaptor methodsFor: 'initialize-release' stamp: 'pmm 11/8/2008 13:01'!
initialize
	super initialize.
	site := WAPlugableSite named: (WAExternalID new: 32) seasideString on: self.
	site ip: '*'.
	site host: '*'.
	SwazooServer singleton addSite: site! !

!WASwazooAdaptor methodsFor: 'private-response-creation' stamp: 'pmm 11/6/2008 14:55'!
internalErrorResponseFor: anError
	^HTTPResponse internalServerError
		entity: 'Internal Error';
		yourself! !

!WASwazooAdaptor methodsFor: 'private-testing' stamp: 'pmm 11/6/2008 14:39'!
isImplemented: aMethodName
	^#('GET' 'POST') includes: aMethodName! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:49'!
isRunning
	^site isServing! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:51'!
isStarting
	^false! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:50'!
isStopped
	^site isServing not! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:50'!
isStopping
	^false! !

!WASwazooAdaptor methodsFor: 'private-response-creation' stamp: 'pmm 11/6/2008 14:39'!
notImplementedResponse
	^HTTPResponse notImplemented! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:57'!
postFieldsOf: aSwazooRequest into: aDictionaryMerger
	"break encapsulation"
	aSwazooRequest ensureFullRead.
	aSwazooRequest postData keysAndValuesDo: [ :key :value |
		| previousData data |
		data := value filename isEmptyOrNil
			ifTrue: [ self decodeUrl: value value ]
			ifFalse: [
				WAFile new
					fileName: (self decodeUrl: value filename);
					contentType: value contentType seasideMimeType;
					contents: value value;
					yourself ].
		aDictionaryMerger at: key put: data ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:57'!
remoteAddressOf: aSwazooRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aSwazooRequest peer.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:46'!
shutDown
	SwazooServer stop! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:46'!
startUp
	SwazooServer stop! !

!WASwazooAdaptor methodsFor: 'accessing' stamp: 'pmm 11/8/2008 12:34'!
statusString
	^self isRunning
		ifTrue: [ 'running' ]
		ifFalse: [ 'stopped' ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:41'!
urlFrom: aSwazooRequest
	^(WAUrl absolute: aSwazooRequest uri printString) decodedWith: self codec! !

!WAPackage class methodsFor: '*Seaside-Adaptors-Swazoo' stamp: 'pmm 11/8/2008 13:24'!
seasideAdaptorsSwazoo
	^ self new
		name: 'Seaside-Adaptors-Swazoo';
		description: 'Makes Seaside run on Swazoo 2';
		addDependency: 'Seaside-Swazoo';
		yourself! !
